#
# AttachmentFile.py
#
# Clase para almacenar un archivo adjunto y sus cabeceras
# correspondientes.
#
# Alejandro Álvarez Ayllón - 28/10/2006
#
from ZODB.PersistentMapping import PersistentMapping
from Persistence import Persistent

from AccessControl import ClassSecurityInfo

class AttachmentFile(Persistent):
    """
    Clase AttachmentFile.
    Almacena un archivo subido y sus propiedades (nombre, tipo
    de contenido, ...)
    """

    security = ClassSecurityInfo()

    def __init__(self, upload = None):
        """
        Descripción: Inicializar con un archivo subido.
        Parámetros:  upload = Tipo FileUpload. Puede ser ignorado.
        Devuelve :   Nada.
        """

        if upload is None:
            return

        self.headers  = PersistentMapping(upload.headers)
        self.filename = upload.filename
        self.data     = upload.read()

    security.declarePublic('getContentType')
    def getContentType(self):
        """
        Descripción: Devolver el tipo MIME.
        Parámetros:  Ninguno.
        Devuelve:    Una cadena con el tipo de contenido.
        """

        if self.headers is None:
            return None
        else:
            return self.headers['content-type']

    security.declarePublic('getFilename')
    def getFilename(self):
        """
        Descripción: Devolver el nombre del archivo.
        Parámetros:  Niguno.
        Devuelve:    Una cadena con el nombre del archivo.
        """
        return self.filename

    security.declarePublic('send')
    def send(self, response):
        """
        Descripción: Manda por un objeto RESPONSE el archivo.
        Parámetros:  response = Tipo RESPONSE. Por él se escribirá el archivo.
        Devuelve:    Nada.
        """

        response.setHeader('Content-Type', self.getContentType())
        response.setHeader('Content-Disposition',
                           'inline; filename="%s"' % self.getFilename())

        response.write(self.data)
