#
# ATCVSFile.py
#
# Para descargar diréctamente del repositorio
#
# Alejandro Álvarez Ayllón - 29/04/2006
#

from Products.Archetypes.public import BaseSchema, Schema
from Products.Archetypes.public import StringField, BooleanField, ReferenceField
from Products.Archetypes.public import BooleanWidget, StringWidget, ReferenceWidget
from Products.Archetypes.public import BaseContent, registerType
from Products.CMFCore import CMFCorePermissions

from AccessControl import ClassSecurityInfo

import Utils

from config import PROJECTNAME, I18N_DOMAIN

# Campos
schema = Utils.ContentSchema +  Schema((
    ReferenceField('project',
                   relationship  = 'From',
                   required      = 1,
                   allowed_types = ('Project',),
                   widget        = ReferenceWidget(label = 'Project',
                                                   label_msgid = 'project',
                                                   i18n_domain = I18N_DOMAIN,
                                                   ),
                   edit_accessor  = 'getParentProject',
                   ),
    StringField('file',
                searchable = 0,
                required   = 0,
                widget     = StringWidget(label       = 'File',
                                          label_msgid = 'file',
                                          i18n_domain = I18N_DOMAIN,
                                          ),
                ),
    StringField('revision',
                searchable = 0,
                required   = 1,
                widget     = StringWidget(label       = 'Revision',
                                          label_msgid = 'revision',
                                          i18n_domain = I18N_DOMAIN,
                                          ),
                ),
    BooleanField('directory',
                 searchable = 0,
                 required   = 1,
                 widget     = BooleanWidget(label       = 'Directory',
                                            label_msgid = 'directory',
                                            i18n_domain = I18N_DOMAIN,
                                            ),
                 ),
    ),)

# Clase
class ATCVSFile(BaseContent, Utils.Content):
    """CVS File"""

    schema = schema

    content_icon    = 'cvs.gif'
    meta_type       = 'ATCVSFile'
    portal_type     = 'CVSFile'
    archetype_name  = 'CVS File'
    typeDescription = 'CVS File'

    actions = ({
        'id'         : 'view',
        'name'       : 'View',
        'action'     : 'string:${object_url}/cvsfile',
        'permissions': (CMFCorePermissions.View,)
        },
               )

    security = ClassSecurityInfo()

    # Devuelve la URL para descargar el archivo desde el repositorio
    security.declareProtected(CMFCorePermissions.View, 'getDownloadUrl')
    def getDownloadUrl(self):
        """
        Descripción: Devuelve la URL para descargar el archivo desde el repositorio.
        Parámetros:  No tiene.
        Devuelve:    La URL para descargar el archivo.
        """
        
        is_dir = self.directory
        path = self.file

        # Necesitamos el nombre y la URL del proyecto
        project      = self.getReferenceImpl(self.getField('project').relationship)[0].getTargetObject()
        project_name = project.unix_name
        project_url  = project.absolute_url()

        # Si no especifica qué directorio, presupongo que es la raíz
        if path == '' or path == '/':
            is_dir = 1
            path = 'root'
            
        if is_dir:
            # Es un directorio completo (en tar.gz)
            url = '%s/project_repository/%s/%s.tar.gz?view=tar&rev=%s' % (project_url, project_name, path, self.revision)
        else:
            # Es un archivo
            url = '%s/project_repository/*checkout*/%s/%s?rev=%s' % (project_url, project_name,
                                                                     path, self.revision)

        # Devolvemos la URL desde donde se descargará
        return url


    # Devuelve el proyecto padre si hay
    security.declareProtected(CMFCorePermissions.View, 'getParentProject')
    def getParentProject(self):
        """
        Descripción: Devuelve el proyecto padre si se encuentra. Se usa para establecer
                     el valor predeterminado de la relación From.
                     Si ya está asociado con algún proyecto, devuelve ese.
        Parámetros:  Niguno.
        Devuelve:    La referencia al proyecto padre o None, según proceda.

        """
        if len(self.getReferenceImpl(self.getField('project').relationship)) == 0:
            # Buscar el proyecto padre y asignarlo si hay
            i = self
            while i and i.portal_type != 'Plone Site':
                if i.portal_type == 'Project':
                    # Lo encontramos, establecemos el valor del campo
                    self.getField('project').set(self, i)
                    # Y salimos del bucle
                    break

                i = i.getParentNode()

        # Devolvemos el valor del campo
        return self.getField('project').getRaw(self)
        

# Registrar el tipo
registerType(ATCVSFile)
