#
# ATProjectSuggestion.py
#
# Sugerencia de proyecto
#
# Alejandro Álvarez Ayllón - 10/09/2006
#
#

from Products.Archetypes.public import BaseSchema, Schema
from Products.Archetypes.public import StringField, TextField, DateTimeField, LinesField, IntegerField
from Products.Archetypes.public import StringWidget, RichWidget, CalendarWidget, LinesWidget, IntegerWidget
from Products.Archetypes.public import SelectionWidget
from Products.Archetypes.public import BaseContent, registerType
from Products.CMFCore           import CMFCorePermissions
from Products.CMFCore.utils     import getToolByName

from ZODB.PersistentMapping import PersistentMapping
from ZODB.PersistentList    import PersistentList

from AccessControl import ClassSecurityInfo, getSecurityManager

from config import PROJECTNAME, I18N_DOMAIN, CENTRES

import Utils
from AttachmentFile import AttachmentFile

import os
import popen2
import re

from Globals import package_home

# Campos
schema = Utils.ContentSchema + Schema((
    StringField('teacher',
                searchable = 1,
                required   = 1,
                widget     = StringWidget(label       = 'Teacher',
                                          label_msgid = 'teacher',
                                          i18n_domain = I18N_DOMAIN,
                                          size        = 50,
                                          ),
                ),
    StringField('career',
                searchable = 1,
                required   = 1,
                widget     = StringWidget(label       = 'Career',
                                          label_msgid = 'career',
                                          i18n_domain = I18N_DOMAIN,
                                          size        = 50,
                                          ),
                ),
    StringField('department',
                searchable = 1,
                required   = 1,
                widget     = StringWidget(label       = 'Department',
                                          label_msgid = 'department',
                                          i18n_domain = I18N_DOMAIN,
                                          size        = 60,
                                          ),
                ),
    StringField('centre',
                searchable = 1,
                required   = 1,
                vocabulary = CENTRES,
                widget     = SelectionWidget(label       = 'centre',
                                             label_msgid = 'centre',
                                             i18n_domain = I18N_DOMAIN,
                                             format      = 'select',
                                             ),
                ),
    TextField('long_description',
              searchable = 1,
              required   = 1,
              widget     = RichWidget(label       = 'Long description',
                                      label_msgid = 'long_description',
                                      i18n_domain = I18N_DOMAIN,
                                      ),
              ),
    DateTimeField('end_date',
                  searchable = 0,
                  required   = 1,
                  mutator    = 'reopen',
                  widget     = CalendarWidget(label       = 'End date',
                                              label_msgid = 'end_date',
                                              i18n_domain = I18N_DOMAIN,
                                              ),
                  ),
    IntegerField('posts',
                 searchable = 0,
                 required   = 1,
                 mutator    = 'setPosts',
                 widget     = IntegerWidget(label       = 'Posts',
                                            label_msgid = 'posts',
                                            i18n_domain = I18N_DOMAIN,
                                            ),
                 ),
    ),)

class ATProjectSuggestion(BaseContent, Utils.Content):
    """Project Suggestion"""

    schema = schema

    content_icon    = 'ps_assigned.gif'
    meta_type       = 'ATProjectSuggestion'
    portal_type     = 'ProjectSuggestion'
    archetype_name  = 'Project Suggestion'
    typeDescription = 'Project Suggestion'

    actions = ({
        'id'         : 'view',
        'name'       : 'View',
        'action'     : 'string:${object_url}/ps_view',
        'permissions': (CMFCorePermissions.View,),
        },
                )

    security = ClassSecurityInfo()

    workflow = 'projectsuggestion_workflow'

    pdfTex  = os.path.join( package_home ( globals() ), 'docs', 'ProjectSuggestion.tex')
    htmlDoc = os.path.join( package_home ( globals() ), 'docs', 'ProjectSuggestion.html')
    tmpDir  = '/tmp/'

    # Inicialización
    def __init__(self, oid, **kwargs):
        """
        Descripción: Inicializa el objeto.
        Parámetros:  oid      = El id del objeto.
                     **kwargs = Mantenido por compatibilidad. No se usa a este nivel.
        Devuelve:    Nada.
        """

        # Inicialización de la clase base
        BaseContent.__init__(self, oid, **kwargs)

        # Solicitudes: Una lista con los IDs de los usuarios que piden el proyecto
        self.requests = PersistentList()
        # Asignaciones: Un diccionario donde la clave es el ID del usuario y el valor
        # el archivo subido como adjunto.
        self.assigned = PersistentMapping()

    # Devuelve la fecha de finalización formateada
    security.declareProtected(CMFCorePermissions.View, 'getEndDateString')
    def getEndDateString(self):
        """
        Descripción: Devuelve la fecha de finalización formateada.
        Parámetros:  Ninguno.
        Devuelve:    La fecha de finalización formateada."""

        return self.end_date.strftime('%A %d de %B de %Y a las %H:%M')


    # Un alumno solicita el proyecto
    security.declareProtected(CMFCorePermissions.View, 'request')
    def request(self, REQUEST = None):
        """
        Descripción: Solicita la asignación del proyecto.
        Parámetros:  REQUEST = Opcional. Si se pasa y no se ha iniciado sesión se
                     redirige al formulario de entrada.
        Devuelve:    Nada.
        """

        user = getSecurityManager().getUser()

        # No puede ser anónimo
        if user.getId() is None:
            if not REQUEST is None:
                REQUEST.response.redirect('login_form')
                return
            else:
                raise SystemError('Anonymous user')

        # Nombre
        uid = user.getId()

        # Se guarda el nombre del alumno
        # No se permite solicitar al creador ni a alguien que ya ha solicitado.
        if uid not in self.requests and \
           uid not in self.assigned.keys() and \
           uid != self.Creator():
            self.requests.append(uid)
 
    # Devuelve una lista de los solicitantes como diccionarios
    security.declareProtected(CMFCorePermissions.View, 'getRequests')
    def getRequests(self):
        """
        Descripción: Devuelve una lista de los solicitantes.
        Parámetros:  Ninguno.
        Devuelve:    Una lista donde cada elemento es un diccionario con algunos
                     datos de los solicitantes: id de usuario (id), nombre real (name)
                     y e-mail (mail).
        """

        membership_tool = getToolByName(self, 'portal_membership')

        alumDict = []

        for i in self.requests:
            member = membership_tool.getMemberById(i)
            if member is None:
                raise KeyError('Invalid member id')

            fullname = member.getProperty('fullname')
            if fullname == '':
                fullname = i

            alumDict.append({'id'  : i,
                             'name': fullname,
                             'mail': member.getProperty('email')
                             })

        return alumDict

    # Devuelve una lista de los asignados como diccionarios
    security.declareProtected(CMFCorePermissions.View, 'getAssigned')
    def getAssigned(self):
        """
        Descripción: Devuelve una lista de los asignados como diccionarios.
        Parámetros:  Ninguno.
        Devuelve:    Una lista donde cada elemento es un diccionario con algunos
                     datos de los asignados: id de usuario (id), nombre real (name),
                     e-mail (mail) y si han subido el anexo (has_attachment).
        """

        membership_tool = getToolByName(self, 'portal_membership')

        alumDict = []

        for i in self.assigned.keys():
            member = membership_tool.getMemberById(i)
            if member is None:
                raise KeyError('Invalid member id')

            fullname = member.getProperty('fullname')
            if fullname == '':
                fullname = i

            alumDict.append({'id'            : i,
                             'name'          : fullname,
                             'mail'          : member.getProperty('email'),
                             'has_attachment': not(self.assigned[i] == None)
                             })

        return alumDict

    # Devuelve los IDs de los asignados
    security.declareProtected(CMFCorePermissions.View, 'getAssignedIDs')
    def getAssignedIDs(self):
        """
        Descripción: Devuelve los IDs de los asignados.
        Parámetros:  Ninguno.
        Devuelve:    Una lista con los IDs de los usuarios asignados.
        """
        
        return self.assigned.keys()

    # Indica si puede asignarle el proyecto a alguien
    security.declareProtected(CMFCorePermissions.View, 'canAssign')
    def canAssign(self):
        """
        Descripción: Indica si puede asignarle el proyecto a alguien.
        Parámetros:  Ninguno.
        Devuelve:    True si se puede asignar alumnos. False si no. (Depende
                     del estado del proyecto, de las plazas cubiertas, de los
                     permisos, ...)
        """

        return getSecurityManager().checkPermission(CMFCorePermissions.ModifyPortalContent, self) \
               and \
               self.getStatus() == 'closed' \
               and \
               len(self.assigned) < self.posts \
               and \
               len(self.requests) > 0

    # Indica si se puede sacar del proyecto a alguien
    security.declareProtected(CMFCorePermissions.View, 'canUnassign')
    def canUnassign(self):
        """
        Descripción: Indica si se puede sacar del proyecto a alguien.
        Parámetros:  Niguno.
        Devuelve:    True si se puede deasignaar alumnos. False si no. (Depende
                     de los permisos y de los alumnos asignados)
        """

        return getSecurityManager().checkPermission(CMFCorePermissions.ModifyPortalContent, self) \
               and \
               len(self.assigned) > 0


    # Asignar a partir de un formulario
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'assignFromForm')
    def assignFromForm(self, form):
        """
        Descripción: Asignar a partir de un formulario.
        Parámetros:  form = El formulario (un diccionario). Asignará el proyecto
                     a los usuarios pasados como valor de los campos que empiecen por 'alum'.
        Devuelve:    Nada.
        """

        for i in form.keys():
            if i[0:4] == 'alum' and len(self.assigned) < self.posts:
                self.assign(form[i])

    # Eliminar alumnos de la lista de asignados a partir de un formulario
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'unassignFromForm')
    def unassignFromForm(self, form):
        """
        Descripción: Eliminar alumnos a partir de un formulario.
        Parámetros:  form = El formulario (un diccionario). Se deasignaran los usuarios
                     pasados como valor de los campos que empiecen por 'alum'.
        Devuelve:    Nada.
        """

        for i in form.keys():
            if i[0:4] == 'alum':
                self.unassign(form[i])
    
    
    # Asignar el proyecto
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'assign')
    def assign(self, uid):
        """
        Descripción: Se le asigna el proyecto al solicitante.
        Parámetros:  uid = El ID del usuario que se asignará al proyecto.
        Devuelve:    Nada. Lanza una excepción si no hay plazas.
        """
        

        # Comprobar que no superamos las plazas
        if len(self.assigned) >= self.posts:
            raise ValueError('All posts are covered')

        # Eliminamos de solicitudes
        self.requests.remove(uid)

        # Y metemos en asignados
        if uid not in self.assigned.keys():
            self.assigned[uid] = None

        # Si están cubiertas todas las plazas, cambiamos estado
        if len(self.assigned) >= self.posts:
            wf_tool = getToolByName(self, 'portal_workflow')
            wf_tool.doActionFor(self, 'assign', self.workflow)

    # Eliminar un alumno asignado
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'unassign')
    def unassign(self, uid):
        """
        Descripción: Eliminar un alumno asignado.
        Parámetros:  uid = El ID del usuario que se deasignará.
        Devuelve:    Nada.
        """

        del self.assigned[uid]

        # Cambiamos el estado
        wf_tool = getToolByName(self, 'portal_workflow')
        wf_tool.doActionFor(self, 'unassign', self.workflow)

    # Si cambia el número de plazas, ajusta el estado
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setPosts')
    def setPosts(self, value):
        """
        Descripción: Si aumenta el número de plazas estando asignado, cambiar el estado.
                     Si disminuyen el número de plazas estando cerrado (y el cupo cubierto ahora),
                     asignar.
        Parámetros:  value = Nuevo valor.
        Devuelve:    Nada.
        """
        wf_tool = getToolByName(self, 'portal_workflow')

        value = int(value) # Lo pasamos a entero

        if value < self.posts and self.getStatus() == 'closed':
            # Ha disminuido el número de plazas estando cerrado, igual ya deberíamos
            # dar por asignado.
            if len(self.assigned) >= value:
                wf_tool.doActionFor(self, 'assign', self.workflow)
        elif value > self.posts and self.getStatus() == 'assigned':
            # Ha aumentado el número de plazas estando asignado, igual deberíamos
            # volver a cerrado
            if len(self.assigned) < value:
                wf_tool.doActionFor(self, 'unassign', self.workflow)

        # Guardamos el valor
        self.posts = value                

    # Vuelve a abrir el proyecto si la fecha ha cambiado
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'reopen')
    def reopen(self, value):
        """
        Descripción: Reabre el proyecto si la fecha ha cambiado.
        Parámetros:  value = La nueva fecha máxima.
        Devuelve:    Nada.
        """

        from DateTime import DateTime
        from datetime import datetime

        # Herramienta para el workflow
        wf_tool = getToolByName(self, 'portal_workflow')
        wf_info = wf_tool.getStatusOf(self.workflow, self)

        # Asignamos el nuevo valor
        self.end_date = DateTime(value)

        # Si la nueva fecha es mayor que la actual, reabrir
        if wf_info['state'] == 'closed' and datetime.now() < datetime.fromtimestamp(self.end_date):
            wf_tool.doActionFor(self, 'reopen', self.workflow)


    # Devueve el estado del objeto en el workflow
    security.declareProtected(CMFCorePermissions.View, 'getStatus')
    def getStatus(self):
        """
        Descripción: Devuelve el estado del objeto.
        Parámetros:  Ninguno.
        Devuelve:    El estado de la propuesta.
        """

        from datetime import datetime

        # Herramienta para el workflow
        wf_tool = getToolByName(self, 'portal_workflow')

        # Obtenemos el estado actual
        wf_info = wf_tool.getStatusOf(self.workflow, self)


        if wf_info is not None and self.end_date is not None:
            # Comprobamos si hay que cerrar
            if wf_info['state'] == 'suggested' and datetime.fromtimestamp(self.end_date) < datetime.now():

                #Cerramos
                wf_tool.doActionFor(self, 'close', self.workflow)

                # Si están las plazas cubiertas (pueden estarlo de antes)
                # pasar a asignado
                if len(self.assigned) >= self.posts:
                    wf_tool.doActionFor(self, 'assign', self.workflow)

                #Volvemos a obtener el estado
                wf_tool.getStatusOf(self.workflow, self)
                
            # Devolvemos el estado
            return wf_info['state']
        else:
            return 'suggested'

    # Sobrecarga la función getIcon para que cambie según el estado
    security.declareProtected(CMFCorePermissions.View, 'getIcon')
    def getIcon(self):
        """
        Descripción: Devolver el nombre del icono en función del estado.
        Parámetros: Ninguno.
        Devuelve:   El nombre del icono correspondiente al estado.
        """

        return 'ps_%s.gif' % self.getStatus()

    # Sube el adjunto correspondiente al usuario actual (o lanza excepción)
    security.declareProtected(CMFCorePermissions.View, 'uploadAttachment')
    def uploadAttachment(self, attachment):
        """
        Descripción: Sube el adjunto del usuario actual si es uno de los asignados.
        Parámetros:  attachment = Tipo FileUpload. Archivo subido.
        Devuelve:    Nada.
        """

        # Usuario actual
        user = getSecurityManager().getUser().getId()

        # Si no es uno de los asignados, error
        if user not in self.assigned.keys():
            raise KeyError('Invalid user')

        # Asignarlo
        self.assigned[user] = AttachmentFile(attachment)

    # Devolver el adjunto pedidio
    security.declareProtected(CMFCorePermissions.View, 'getAttachment')
    def getAttachment(self, REQUEST = None):
        """
        Descripción: Escribe el archivo adjunto del usuario pedido por la salida (response).
        Parámetros:  REQUEST = El objeto request de la petición.
        Devuelve:    Nada. Escribe por REQUEST.response el archivo.
        """

        if REQUEST is None:
            return

        for i in REQUEST.items():
            if i[0] == 'id':
                user = i[1]

        # Escribir por la salida
        if user in self.assigned.keys():
            self.assigned[user].send(REQUEST.response)
        else:
            REQUEST.response.redirect(self.absolute_url())

    # Genera y envia el documento PDF
    security.declareProtected(CMFCorePermissions.View, 'getPdfForm')
    def getPdfForm(self, REQUEST = None):
        """
        Descripción: Genera y envia el documento PDF.
        Parámetros:  REQUEST = El objeto request de la petición.
        Devuelve:    Nada. Escribe por REQUEST.response el PDF.
        """
  
        if REQUEST is None:
            return
        
        response = REQUEST.response
        membership_tool = getToolByName(self, 'portal_membership')
        plone_tool      = getToolByName(self, 'plone_utils')
        forja_tool      = getToolByName(self, 'forja_tool')

        # Comprobar si se pueden generar PDF
        if not forja_tool.generate_pdf_forms:
            raise EnvironmentError('Cannot generate PDF forms')

        # Temporal sin extensión
        tmpPathWE = '%s%s' % (self.tmpDir, self.id)
        
        # Diccionario con las variables
        alum = ''
        for i in REQUEST.items():
            if i[0] == 'id':
                alum = i[1]

        member = membership_tool.getMemberById(alum)
        if member is None:
            fullname = 'Nombre de alumno incorrecto'
        elif alum not in self.assigned.keys():
            fullname = 'Este proyecto no se le ha asignado'
        else:
            fullname = unicode(member.getProperty('fullname'), 'utf-8')


        docDict = {'@@'         : '@',
                   '@UCA_PNG'   : os.path.join(package_home(globals()),
                                               'docs',
                                               '%s.png' % plone_tool.normalizeString(self.centre)
                                               ),
                   '@STUDENT'   : fullname,
                   '@CAREER'    : self.career,
                   '@DEPARTMENT': self.department,
                   '@CENTRE'    : self.centre,
                   '@TEACHER'   : self.teacher,
                   '@TITLE'     : self.title}

        # Abrimos el archivo, y generamos el temporal cambiando las variables
        tex = open(self.pdfTex, 'r')
        tmp = open('%s.tex' % tmpPathWE, 'w')

        param = re.compile(u'@[\w@]+')

        for i in tex:
            m = param.search(i)
            while m is not None:
                # Cambiar un parámetro por su valor
                i = i[0:m.start()] + docDict[m.group()].encode('latin-1') + i[m.end():]

                m = param.search(i)
                
            # Escribir al fichero
            tmp.write(i)

        tex.close()
        tmp.close()
        

        # Cambiamos el directorio de trabajo al directorio temporal
        # (recordando donde estábamos)
        wd = os.getcwd()
        os.chdir(self.tmpDir)

        # Ejecutamos pdflatex
        pdflatex = popen2.Popen3('pdflatex %s.tex' % tmpPathWE,
                                 True)
        if pdflatex.wait() != 0:
            raise IOError(pdflatex.childerr.read())

        # Volvemos al directorio de trabajo original
        os.chdir(wd)

        # Devolver el PDF
        response.setHeader('Content-Type', 'application/pdf')
        response.setHeader('Content-Disposition', 'inline; filename="propuesta.pdf"')
            
        pdf = open('%s.pdf' % tmpPathWE, 'rb')

        for i in pdf:
            response.write(i)

        pdf.close()

        # Borrar los archivos temporales
        exts = ('pdf', 'tex', 'log', 'aux')

        for i in exts:
            os.unlink('%s.%s' % (tmpPathWE, i))

    # Genera y envía el formulario en HTML
    security.declareProtected(CMFCorePermissions.View, 'getHtmlForm')
    def getHtmlForm(self, REQUEST = None):
        """
        Descripción: Genera y envia el documento HTML.
        Parámetros:  REQUEST = El objeto request de la petición.
        Devuelve:    Nada. Escribe por REQUEST.response el HTML.
        """
  
        if REQUEST is None:
            return
        
        response = REQUEST.response
        membership_tool = getToolByName(self, 'portal_membership')
        plone_tool      = getToolByName(self, 'plone_utils')
        forja_tool      = getToolByName(self, 'forja_tool')

        # Comprobar que se pueden generar HTML
        if not forja_tool.generate_html_forms:
            raise EnvironmentError('Cannot generate HTML forms')

        # Diccionario con las variables
        alum = ''
        for i in REQUEST.items():
            if i[0] == 'id':
                alum = i[1]

        member = membership_tool.getMemberById(alum)
        if member is None:
            fullname = 'Nombre de alumno incorrecto'
        elif alum not in self.assigned.keys():
            fullname = 'Este proyecto no se le ha asignado'
        else:
            fullname = unicode(member.getProperty('fullname'), 'utf-8')


        docDict = {'@@'          : '@',
                   '@UCA_PNG'    : 'getHeaderPng',
                   '@UCA_PNG_ALT': self.centre,
                   '@STUDENT'    : fullname,
                   '@CAREER'     : self.career,
                   '@DEPARTMENT' : self.department,
                   '@CENTRE'     : self.centre,
                   '@TEACHER'    : self.teacher,
                   '@TITLE'      : self.title}

        # Abrimos el archivo, y vamos cambiando las variables y enviando
        html = open(self.htmlDoc, 'r')

        param = re.compile(u'@[\w@]+')

        response.setHeader('Content-Type', 'text/html')

        for i in html:
            m = param.search(i)
            while m is not None:
                # Cambiar un parámetro por su valor
                i = i[0:m.start()] + docDict[m.group()].encode('utf-8') + i[m.end():]

                m = param.search(i)
                
            # Escribir
            response.write(i)

        html.close()

    # Obtener la imágen de la cabecera
    security.declareProtected(CMFCorePermissions.View, 'getHeaderPng')
    def getHeaderPng(self, REQUEST = None):
        """
        Descripción: Escribe por REQUEST.response el archivo PNG con la cabecera asociada al centro.
        Parámetros:  REQUEST = El objeto request de la petición.
        Devuelve:    Nada. Escribe por REQUEST.response el contenido del PNG.
        """

        if REQUEST is None:
            return

        plone_tool = getToolByName(self, 'plone_utils')
        forja_tool = getToolByName(self, 'forja_tool')

        # Comprobar que se puede
        if not forja_tool.generate_html_forms:
            raise EnvironmentError('Cannot generate html forms')

        # Enviar
        png_path   = os.path.join( package_home( globals() ),
                                  'docs',
                                  '%s.png' % plone_tool.normalizeString(self.centre)
                                  )

        png = open(png_path, 'r')

        REQUEST.response.setHeader('Content-Type', 'image/png')
        REQUEST.response.write(png.read())

        png.close()
        

# Registrar el tipo
registerType(ATProjectSuggestion, PROJECTNAME)
